#!/bin/bash
# Script to create a waltz user, generate a key-pair and add the host IP
# for administration

set -eou pipefail

# Wait until FQDN of host is resolvable; otherwise service won't start
# This is used mainly in EAP where there is a delay between reboot and DNS registration for our hostname
# being available
while true; do
    check_dns_fqdn_resolution=$(nslookup ${HOSTNAME} | grep "Name:")
    re='Name.*'${HOSTNAME}'.*'
    if [[ ${check_dns_fqdn_resolution} =~ ${re} ]]; then
        break
    else
        sleep 10
    fi
done

WALTZ_USER=waltz

# Check if the user exists
if id ${WALTZ_USER} > /dev/null 2>&1; then
    echo "user exists"
else
    # Create a local user named waltz if it doesn't exist.
    useradd -c "Waltz User" --create-home -s /bin/bash ${WALTZ_USER}
fi

# Get the home directory of the user
WALTZ_HOME=$(eval echo ~${WALTZ_USER})

# Etcd directory storing certs and persistent data
ETCD_DIR=${WALTZ_HOME}/etcd

mkdir -p ${ETCD_DIR}/data
mkdir -p ${ETCD_DIR}/certs

# etcd link name specified in docker-compose. This is the host waltz will try to
# contact inside the container.
ETCD_LINK_NAME=etcd0

# Generate a self-signed TLS cert for securing waltz/etcd communication.
openssl req -x509 -nodes -newkey rsa:2048 \
        -keyout "${ETCD_DIR}/certs/etcd0-selfsigned.key" \
        -out "${ETCD_DIR}/certs/etcd0-selfsigned.crt" \
        -days 365 \
        -subj "/CN=${ETCD_LINK_NAME}/O=Microsoft/C=US/ST=WA"

# Create .ssh directory if it doesn't exist
mkdir -p ${WALTZ_HOME}/.ssh

# Generate a SSH Key pair for the container to use for administering the host
PKEY=${WALTZ_HOME}/waltz_id_rsa
echo -e 'y\n' | ssh-keygen -t rsa -N "" -f ${PKEY}

# Move the public key to the user
mv ${WALTZ_HOME}/waltz_id_rsa.pub ${WALTZ_HOME}/.ssh/authorized_keys

# Set correct file permissions for ssh
chmod 600 ${PKEY}
chown -R waltz ${WALTZ_HOME}/.ssh
chmod 700 ${WALTZ_HOME}/.ssh
chmod 600 ${WALTZ_HOME}/.ssh/authorized_keys

# Set to ask no password
echo "waltz ALL=(ALL) NOPASSWD: ALL" | sudo tee /etc/sudoers.d/waltz_override

# Get the host ip and put it in hosts file for Ansible to identify the host to
# administer
ip route get 1 | awk '{print $NF;exit}' > ${WALTZ_HOME}/hosts

